--  Dynamic 6 player Finn's Revenge
--  By Strand
--  MDStrand@gmail.com
--  I might not check that e-mail address for quite some time, best way to reach me is through
--  the multiplayer client.  Please feel free to distribute and modify this script.  I would
--  appreciate proper credit, but in the end that's really up to you.  I did my best to document
--  what I've done, please ask questions in the Map comment section or through e-mail.  I do
--  not participate in the Official Forums nor will I.

local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import('/lua/ScenarioFramework.lua')

--  Where we set and remember who and how many people are on which team
local northArmy = {
	army1 = 'ARMY_4',
	army2 = 'ARMY_5',
	army3 = 'ARMY_6',
	active1 = false,
	active2 = false,
	active3 = false,
	active = 0,
}
local southArmy = {
	army1 = 'ARMY_1',
	army2 = 'ARMY_2',
	army3 = 'ARMY_3',
	active1 = false,
	active2 = false,
	active3 = false,
	active = 0,
}

--  Function checks to see if any of the Armies listed in either northArmy or southArmy are
--  in the game.  If so, we then set that army to active and increment the active number of
--  armies for that team.  Function must be called before reSetArmies.
checkArmies = function()    
	--  activeArmy is a string captured from ListArmies.  During the for loop it will list
	--  every active army once once.  A comparison is then done to see if it matches
	--  any of the armies I listed in either southArmy or northArmy.
	for i, activeArmy in ListArmies() do
		if (activeArmy == northArmy.army1) then
			northArmy.active1 = true
			northArmy.active = northArmy.active + 1
		elseif (activeArmy == northArmy.army2) then
			northArmy.active2 = true
			northArmy.active = northArmy.active + 1
		elseif (activeArmy == northArmy.army3) then
			northArmy.active3 = true
			northArmy.active = northArmy.active + 1
		elseif (activeArmy == southArmy.army1) then
			southArmy.active1 = true
			southArmy.active = southArmy.active + 1
		elseif (activeArmy == southArmy.army2) then
			southArmy.active2 = true
			southArmy.active = southArmy.active + 1
		elseif (activeArmy == southArmy.army3) then
			southArmy.active3 = true
			southArmy.active = southArmy.active + 1
		end
	end
end

--  Function repositions an active player based upon what players were set as active from
--  checkArmies.  Function should be called first thing in OnPopulate.  These positions were
--  first set in the editor, and then copied here.  It didn't have to be done this way.

--  NOTE:  army3 for both sides will take the unused position if there are only 2 players on
--  that side.  Thus army1 and army2 will always start in the same position, while army3 will shift.
reSetArmies = function()
	local markers = ScenarioUtils.GetMarkers()
		
	for i, tblData in pairs(markers) do
		for j, activeArmy in ListArmies() do
			if (tblData.army == activeArmy) then
				if (activeArmy == northArmy.army1) then
					if (northArmy.active == 2) then
						tblData.position = VECTOR3( 371.5, 20.7656, 206.5 )
					elseif (northArmy.active == 1) then
						tblData.position = VECTOR3( 361.5, 20.7012, 156.5 )
					end
				elseif (activeArmy == northArmy.army2) then
					if (northArmy.active == 2) then
						tblData.position = VECTOR3( 301.5, 20.6328, 146.5 )
					elseif (northArmy.active == 1) then
						tblData.position = VECTOR3( 361.5, 20.7012, 156.5 )
					end
				elseif (activeArmy == northArmy.army3) then
					if (northArmy.active == 2) then
						if northArmy.active1 then
							tblData.position = VECTOR3( 301.5, 20.6328, 146.5 )
						else
							tblData.position = VECTOR3( 371.5, 20.7656, 206.5 )
						end
					elseif (northArmy.active == 1) then
						tblData.position = VECTOR3( 361.5, 20.7012, 156.5 )
					end
				elseif (activeArmy == southArmy.army1) then
					if (southArmy.active == 2) then
						tblData.position = VECTOR3( 138.5, 20.7402, 303.5 )
					elseif (southArmy.active == 1) then
						tblData.position = VECTOR3( 148.5, 20.7168, 353.5 )
					end
				elseif (activeArmy == southArmy.army2) then
					if (southArmy.active == 2) then
						tblData.position = VECTOR3( 208.5, 20.6328, 363.5 )
					elseif (southArmy.active == 1) then
						tblData.position = VECTOR3( 148.5, 20.7168, 353.5 )
					end
				elseif (activeArmy == southArmy.army3) then
					if (southArmy.active == 2) then
						if southArmy.active1 then
							tblData.position = VECTOR3( 208.5, 20.6328, 363.5 )
						else
							tblData.position = VECTOR3( 138.5, 20.7402, 303.5 )
						end
					elseif (southArmy.active == 1) then
						tblData.position = VECTOR3( 148.5, 20.7168, 353.5 )
					end
				end
				
				--  ListArmies cannot be called inside InitializeStartLocation if you have
				--  more than 2 players active in the game.  InitializeStartLocation will
				--  reset an army's starting location given that army's string identifier.
				--  For example activeArmy must equal something along the lines of ARMY_1
				ScenarioUtils.InitializeStartLocation(activeArmy)
			end
		end
	end
end

--  Function calls checkArmies and then begins to reposition the resource markers based
--  upon how many players are listed as active in northArmy and southArmy.  Data for these
--  markers is kept in the _save.lua file.  Once again, I placed them in the editor, then copied
--  their positions to the file.  Really boring process, and could honestly be done better than
--  how I did it.  Suggest only setting up alternate locations, and checking to see if any are
--  listed before setting it.

--  NOTE:  All resources markers are used if you fully populate the map.  Otherwise resources
--  will not be generated.  Some markers get moved instead of being unplaced.
function ScenarioUtils.CreateResources()
	local markers = ScenarioUtils.GetMarkers()
	checkArmies()
	
    for i, tblData in pairs(markers) do
        if tblData.resource then
		
			--  By default, every resource should spawn for 3v3.  This is used for
			--  determining whether or not a resource should be on the map considering
			--  the number of players on each side.
			local isOnMap = true
			
			--  A very ridiculous way of doing this.  This is where I determine if the marker
			--  is for the north or south side.  I would of used another variable in the _save.lua
			--  but some of the markers switch sides when they get repositioned, and I did
			-- not want to remap the marker positions.  If someone does this better than me
			--  I will not be inpressed, as this is a terrible way of doing this, as is most of
			-- the stuff here.  The important thing is it works.
			
			--  I'm checking to to see how many armies are active and if so does this marker
			--  have a position when there are that many armies.  There's also a check to see
			--  what side the marker will associate with.  When the marker's z position is less
			--  than 256, that means it's on the top half of the map.  When it is greater it is on
			--  the bottom half.  I had to add to more statements to check if there are 3 armies
			--  active for each side otherwise isOnMap would be set to false when there were
			--  3 people on either side and no resources would be spawned.
			if (northArmy.active == 2 and tblData.two and tblData.two[3] < 256) then
				tblData.position = tblData.two
			elseif (northArmy.active == 1 and tblData.one and tblData.one[3] < 256) then
				tblData.position = tblData.one
			elseif (southArmy.active == 2 and tblData.two  and tblData.two[3] > 256) then
				tblData.position = tblData.two
			elseif (southArmy.active == 1 and tblData.one and tblData.one[3] > 256) then
				tblData.position = tblData.one
			elseif (northArmy.active == 3 and tblData.position[3] < 256) then
			elseif (southArmy.active == 3 and tblData.position[3] > 256) then
			else
				--  Setting to false because the marker failed to meet ALL of the above
				-- restrictions.  The marker will not be spawned.
				isOnMap = false
			end
			
			--  Check to see if this marker will spawn.  Create resource code follows here
			--  copied from it's source in ScenarioUtilities.lua.  Code commented out there
			--  was removed here.
			if isOnMap then
	            CreateResourceDeposit(
	                tblData.type,
	                tblData.position[1], tblData.position[2], tblData.position[3],
	                tblData.size
	            )

	            local albedo, sx, sz, lod
				
	            if tblData.type == "Mass" then
	                albedo = "/env/common/splats/mass_marker.dds"
	                sx = 2
	                sz = 2
	                lod = 100
	                CreatePropHPR(
	                    '/env/common/props/massDeposit01_prop.bp',
	                    tblData.position[1], tblData.position[2], tblData.position[3],
	                    Random(0,360), 0, 0
	                )
	            else
	                albedo = "/env/common/splats/hydrocarbon_marker.dds"
	                sx = 6
	                sz = 6
	                lod = 200
	                CreatePropHPR(
	                    '/env/common/props/hydrocarbonDeposit01_prop.bp',
	                    tblData.position[1], tblData.position[2], tblData.position[3],
	                    Random(0,360), 0, 0
	                )
	            end
				
	            CreateSplat(
	                tblData.position,           # Position
	                0,                          # Heading (rotation)
	                albedo,                     # Texture name for albedo
	                sx, sz,                     # SizeX/Z
	                lod,                        # LOD
	                0,                          # Duration (0 == does not expire)
	                -1 ,                        # army (-1 == not owned by any single army)
	                0
	            )
			end
        end
    end
end

--  Function is necessary for the game to begin.  reSetArmies is called here.
function OnPopulate()	
	reSetArmies()
	ScenarioUtils.InitializeArmies()
end

function OnStart(self)
end
